Attribute VB_Name = "modBoard"
Option Explicit
Option Compare Binary
Option Base 0


' Bitmap info type
Private Type GDIBitmap
    Handle As Long
    SizeX As Long
    SizeY As Long
    FileName As String
End Type

' Bitmap indice enum
Private Enum BitmapEnums
    BMPCell = 0
    BMPX
    BMPO
    BMPObstacle
    BMPHighlight
End Enum


' The picturebox which represents the graphical board
Private BoardObject As PictureBox
Private ResSize As Long

' Main DC where bitmaps get selected onto
Private DrawhDC As Long

' Style settings
Public BoardStyle As String
Public PlayStyleSounds As Boolean

' Style constants
Public Const StyleNone = "Default"


' GDI object data
Private Bitmaps(4) As GDIBitmap


' API declarations
Private Declare Function CreateCompatibleDC Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function DeleteDC Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function LoadImage Lib "user32" Alias "LoadImageA" (ByVal hInst As Long, ByVal lpsz As String, ByVal un1 As Long, ByVal n1 As Long, ByVal n2 As Long, ByVal un2 As Long) As Long
Private Declare Function BitBlt Lib "gdi32" (ByVal hDestDC As Long, ByVal X As Long, ByVal Y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal dwRop As Long) As Long
Private Declare Function StretchBlt Lib "gdi32" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal nSrcWidth As Long, ByVal nSrcHeight As Long, ByVal dwRop As Long) As Long
Private Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
Private Declare Function GetDC Lib "user32" (ByVal hwnd As Long) As Long
Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
Private Declare Function GetObject Lib "gdi32" Alias "GetObjectA" (ByVal hObject As Long, ByVal nCount As Long, lpObject As Any) As Long

' Set the board object
Public Sub SetObject(ByRef picObject As PictureBox)

    ' Ignore non-existent controls
    If picObject Is Nothing Then Exit Sub
    
    ' Create the reference
    Set BoardObject = picObject
    
    ' Delete the old DC and create a new one
    DeleteDC DrawhDC
    DrawhDC = CreateCompatibleDC(BoardObject.hdc)

    If BoardStyle <> StyleNone Then modBoard.LoadStyle BoardStyle

End Sub


' Clear the game board
Public Sub Clear()

    ReDim Board(BoardWidth - 1, BoardHeight - 1) As BoardType

End Sub


' Draw the entire board and occupied cells
Public Sub Draw()

    Dim Column As Integer
    Dim Row As Integer
    
    Dim X1 As Integer
    Dim Y1 As Integer
    Dim X2 As Integer
    Dim Y2 As Integer
    
    
    ' Exit if there is no board object
    If BoardObject Is Nothing Then Exit Sub
    
    
    ' Adjust cell sizes
    ' TODO: These calculations are a bit weird but they work. Fix it.
    CellSize = BoardObject.ScaleHeight / BoardHeight
    CellBorderSize = (CellSize / 32) + 1
        
    ' Clear the board
    BoardObject.BackColor = ColorBackground
    BoardObject.DrawWidth = CellBorderSize
    
    If BoardStyle = StyleNone Then
    
        ' Draw the vertical part of the grid
        For Column = 0 To BoardWidth
    
            X1 = Column * CellSize
            Y1 = 0
            X2 = Column * CellSize
            Y2 = BoardHeight * CellSize
    
            BoardObject.Line (X1, Y1)-(X2, Y2), ColorGrid
    
        Next Column
    
        ' Draw the horizontal part of the grid
        For Row = 0 To BoardHeight
    
            X1 = 0
            Y1 = Row * CellSize
            X2 = BoardWidth * CellSize
            Y2 = Row * CellSize
    
            BoardObject.Line (X1, Y1)-(X2, Y2), ColorGrid
    
        Next Row
        
        
        ' Draw the obstacles
        For Column = 0 To BoardWidth - 1
            For Row = 0 To BoardHeight - 1
                If Board(Column, Row).Occupied = True Then modBoard.DrawCell Column, Row
            Next Row
        Next Column
        
    Else
        
        ' GDI type
        For Column = 0 To BoardWidth - 1
            For Row = 0 To BoardHeight - 1
                If Board(Column, Row).Occupied = True Then modBoard.DrawCell Column, Row Else modBoard.EraseCell Column, Row
            Next Row
        Next Column
    
    End If
    
    ' Draw the winning highlight if there is any
    If WinVector.Valid = True Then modBoard.DrawHighlightSeries WinVector
    
End Sub


' Draw a single cell
Public Sub DrawCell(ByRef CellX As Integer, ByRef CellY As Integer)

    Dim X1 As Integer
    Dim Y1 As Integer
    Dim X2 As Integer
    Dim Y2 As Integer
    Dim Border As Integer
    
    Dim XC As Integer
    Dim YC As Integer
    Dim MarkSize As Integer
        
    
    ' Calculate the cell's bounding box
    X1 = CellX * CellSize
    Y1 = CellY * CellSize
    X2 = (CellX * CellSize) + CellSize
    Y2 = (CellY * CellSize) + CellSize
    Border = (CellBorderSize * 2)
        
    ' Erase the cell
    If BoardStyle = StyleNone Then
        BoardObject.Line (X1, Y1)-(X2, Y2), ColorBackground, BF
        BoardObject.Line (X1, Y1)-(X2, Y2), ColorGrid, B
    Else
        modBoard.EraseCell CellX, CellY
    End If
    
    ' Draw the mark
    Select Case Board(CellX, CellY).Player
        
        ' A cross (X)
        Case Player1
            If BoardStyle = StyleNone Then
                BoardObject.Line (X1 + Border, Y1 + Border)-(X2 - Border, Y2 - Border), ColorX
                BoardObject.Line (X1 + Border, Y2 - Border)-(X2 - Border, Y1 + Border), ColorX
            Else
                SelectObject DrawhDC, Bitmaps(BMPX).Handle
                StretchBlt BoardObject.hdc, X1, Y1, CellSize, CellSize, DrawhDC, 0, 0, ResSize, ResSize, vbSrcCopy
            End If
        
        
        ' A circle (O)
        Case Player2
            If BoardStyle = StyleNone Then
                ' Calculate the center of the cell and size of the mark
                XC = X1 + ((X2 - X1) \ 2)
                YC = Y1 + ((Y2 - Y1) \ 2)
                MarkSize = (CellSize \ 2) - Border
            
                BoardObject.Circle (XC, YC), MarkSize, ColorO
            Else
                SelectObject DrawhDC, Bitmaps(BMPO).Handle
                StretchBlt BoardObject.hdc, X1, Y1, CellSize, CellSize, DrawhDC, 0, 0, ResSize, ResSize, vbSrcCopy
            End If
            
            
        ' Obstacle
        Case Blocked
            If BoardStyle = StyleNone Then
                BoardObject.Line (X1 + Border, Y1 + Border)-(X2 - Border, Y2 - Border), ColorObstacle, BF
            Else
                SelectObject DrawhDC, Bitmaps(BMPObstacle).Handle
                StretchBlt BoardObject.hdc, X1, Y1, CellSize, CellSize, DrawhDC, 0, 0, ResSize, ResSize, vbSrcCopy
            End If
    
    End Select
    
End Sub


' Draw a highlight onto a cell
' NOTE: Color is ignored when using a style
Public Sub DrawHighlight(ByRef CellX As Integer, ByRef CellY As Integer, ByRef Color As Long)

    Dim X1 As Integer
    Dim Y1 As Integer
    Dim X2 As Integer
    Dim Y2 As Integer
    Dim Border As Integer
    
    
    ' Calculate the cell's bounding box
    X1 = CellX * CellSize
    Y1 = CellY * CellSize
    X2 = (CellX * CellSize) + CellSize
    Y2 = (CellY * CellSize) + CellSize
    Border = CellBorderSize
    
    ' Just draw it
    If BoardStyle = StyleNone Then
        BoardObject.Line (X1 + Border, Y1 + Border)-(X2 - Border, Y2 - Border), Color, B
    Else
        SelectObject DrawhDC, Bitmaps(BMPHighlight).Handle
        StretchBlt BoardObject.hdc, X1, Y1, CellSize, CellSize, DrawhDC, 0, 0, ResSize, ResSize, vbSrcAnd
    End If
    
End Sub


' Initialise the obstacles on the board
Public Sub InitObstacles()

    Dim A As Long
    Dim X As Integer
    Dim Y As Integer
    
    
    ' Init a new randomizer seed
    If GameType = GameServer Or GameType = GameLocal Then
        RandomSeed = Timer
        Randomize Timer
    ElseIf GameType = GameClient Then
        RandomSeed = modNet.GetVar(VAR_SEED)
        Randomize RandomSeed
    End If
    
    For A = 0 To Obstacles - 1
    
        ' Generate coordinates that aren't already taken
        Do
            X = Int(Rnd * BoardWidth)
            Y = Int(Rnd * BoardHeight)
        Loop Until Board(X, Y).Occupied = False
                
        ' Mark the cell
        Board(X, Y).Occupied = True
        Board(X, Y).Player = Blocked
    
    Next A

End Sub


' Highlight a winning series
Public Sub DrawHighlightSeries(ByRef Series As EnhancedVectorType)

    Dim A As Long
    
    Dim CX As Integer
    Dim CY As Integer
    
    
    ' Set starting position
    CX = Series.OriginX - Series.X
    CY = Series.OriginY - Series.Y
    
    ' Draw the series
    For A = 1 To WinCells
    
        ' Advance a single vector
        CX = CX + Series.X
        CY = CY + Series.Y
        
        ' Draw the highlight
        modBoard.DrawHighlight CX, CY, ColorHighlight
    
    Next A
    
    BoardObject.Refresh

End Sub


' Load board style graphics
Public Sub LoadStyle(ByRef StyleName As String)

    Dim A As Long
    
    
    ' Setup style filename
    Bitmaps(0).FileName = "cell.bmp"
    Bitmaps(1).FileName = "x.bmp"
    Bitmaps(2).FileName = "o.bmp"
    Bitmaps(3).FileName = "obstacle.bmp"
    Bitmaps(4).FileName = "highlight.bmp"
    

    ' Delete the old bitmaps
    For A = 0 To UBound(Bitmaps())
        
        ' Delete the old handle
        DeleteObject Bitmaps(A).Handle
        
        ' Load the new bitmap
        Bitmaps(A).Handle = LoadImage(App.hInstance, App.Path & "\styles\" & StyleName & "\" & Bitmaps(A).FileName, 0, 0, 0, &H10)
        
        ' Get the size of the bitmap
        
        
    Next A
    
    
    ResSize = 140

End Sub


' Erase a single cell
' NOTE: This one is GDI graphics only
Public Sub EraseCell(ByRef CellX As Integer, ByRef CellY As Integer)

    Dim X1 As Integer
    Dim Y1 As Integer
    Dim X2 As Integer
    Dim Y2 As Integer
    
    
    ' Calculate the cell's bounding box
    X1 = CellX * CellSize
    Y1 = CellY * CellSize
    X2 = (CellX * CellSize) + CellSize
    Y2 = (CellY * CellSize) + CellSize
    
    ' Blit the bitmap
    SelectObject DrawhDC, Bitmaps(BMPCell).Handle
    StretchBlt BoardObject.hdc, X1, Y1, CellSize, CellSize, DrawhDC, 0, 0, ResSize, ResSize, vbSrcCopy

End Sub


' Free used GDI resources
Public Sub FreeResources()

    Dim A As Long
    
    
    ' Delete the bitmaps
    For A = 0 To UBound(Bitmaps())
        DeleteObject Bitmaps(A).Handle
    Next A

    ' Delete the drawing device context
    DeleteDC DrawhDC

End Sub
