Attribute VB_Name = "modGame"
Option Explicit
Option Compare Binary
Option Base 0


' Vector type
Private Type VectorType
    X As Integer
    Y As Integer
End Type

' Enhanced vector type (for return)
Public Type EnhancedVectorType
    X As Integer
    Y As Integer
    Valid As Boolean
    OriginX As Integer
    OriginY As Integer
End Type


' Game error enum
Public Enum GameError
    None = 0
    InvalidCell
End Enum

' Game type enum
Public Enum GameTypeEnum
    GameLocal = 0
    GameServer = 1
    GameClient = 2
End Enum


' Turn settings
Public PlayerTurn As Integer
Public Turn As Integer

' Game settings
Public GameActive As Boolean
Public GameType As Byte
Public MaxCells As Integer
Private CellsFilled As Integer

' Public winning enhanced vector
Public WinVector As EnhancedVectorType


' Start a new game
Public Sub NewGame()

    Dim Dummy As Long
    
    
    ' If in a game, ask if the user is sure
    If GameActive = True Then
        Dummy = MsgBox("Are you sure you want to end the current game?", vbYesNo Or vbQuestion, "T3 Online")
        If Dummy = vbNo Then Exit Sub
    End If
    
    modLog.WriteMsg "** Starting new game..."
    
    ' Reset player info
    ReDim Player(1) As PlayerType
    
    ' Winvector shouldn't be drawn again
    WinVector.Valid = False
    
    ' Display the new game dialog
    frmNewGame.Show 1
    
    ' If playing a netgame, wait for a connection and exchange data
    frmMain.Show
    modNet.Connect
    
    ' Setup the new board
    modBoard.SetObject frmMain.picBoard
    modBoard.Clear
    modBoard.InitObstacles
    modBoard.Draw
    
    ' Reset game stats
    PlayerTurn = Player1
    Turn = 0
    CellsFilled = 0
    
    ' Calculate the number of useable cells
    MaxCells = (BoardWidth * BoardHeight) - (Obstacles - 1) - 1
    
    ' Current game is active
    GameActive = True
    
    modLog.WriteMsg "** Game is now active."
    
    ' Update status indicators
    frmMain.UpdateStatus

End Sub


' Click a cell
Public Function CellClick(ByRef CellX As Integer, ByRef CellY As Integer) As Integer

    Dim SearchResult As EnhancedVectorType
    
    
    ' Default value
    CellClick = GameError.None
    
    ' See if this cell is occupied already or out of board boundaries (don't
    ' OR these together)
    If CellX < 0 Or CellX > BoardWidth - 1 Or CellY < 0 Or CellY > BoardHeight - 1 Then
        CellClick = GameError.InvalidCell
        Exit Function
    End If
    
    If Board(CellX, CellY).Occupied = True Then
        CellClick = GameError.InvalidCell
        Exit Function
    End If
    
       
    ' Mark this cell as occupied
    Board(CellX, CellY).Occupied = True
    Board(CellX, CellY).Player = PlayerTurn
    CellsFilled = CellsFilled + 1
    
    modLog.WriteMsg Player(PlayerTurn).Name & " clicked cell (" & CellX & ", " & CellY & ")"
    
    ' Update the cell
    modBoard.DrawCell CellX, CellY
    frmMain.picBoard.Refresh
    
    ' Play the cell-clicking sound
    modUtil.PlayStyleSound "mark.wav"
    
    ' Do a search for a full row
    SearchResult = FullRowSearch(CellX, CellY)
    If SearchResult.Valid = True Then
        
        ' Highlight the winning series
        modBoard.DrawHighlightSeries SearchResult
        WinVector = SearchResult
        
        modLog.WriteMsg Player(PlayerTurn).Name & " has won the game!"
        
        ' Play the winning sound
        modUtil.PlayStyleSound "win.wav"
        
        ' A player just won the game
        MsgBox Player(PlayerTurn).Name & " won the game!", vbInformation Or vbOKOnly, "T3 Online"
        
        ' The game is now inactive
        GameActive = False
        
        Exit Function
        
    End If
    
    ' See if the board is full, so they lose
    If MaxCells = CellsFilled Then
        
        ' Play the losing sound
        modUtil.PlayStyleSound "lose.wav"
        
        modLog.WriteMsg "The game ended in a tie."
        
        ' Show what happened
        MsgBox "It's a tie!", vbInformation Or vbOKOnly, "T3 Online"
        
        ' The game is now inactive
        GameActive = False
        
        Exit Function
        
    End If
    
End Function


' Advance a turn
Public Sub AdvanceTurn()

    ' Next turn
    Turn = Turn + 1
    
    ' Next player
    PlayerTurn = PlayerTurn + 1
    If PlayerTurn > UBound(Player()) Then PlayerTurn = 0

End Sub


' This algorithm searches in the entire vector's axis (VectorAxis * WinCells)
' Too bad I had to use the dreaded Goto...
Public Function FullRowSearch(ByRef OriginX As Integer, ByRef OriginY As Integer) As EnhancedVectorType

    Dim A As Long
    
    Dim VecX As Integer
    Dim VecY As Integer
    Dim Vector As Integer
    
    Dim Vectors(3) As VectorType
    
    Dim CX As Integer
    Dim CY As Integer
    Dim CMark As Integer
    
    Dim RowCount As Integer
    
   
    ' Setup vector array
    Vectors(0).X = 0
    Vectors(0).Y = -1
    
    Vectors(1).X = 1
    Vectors(1).Y = -1
    
    Vectors(2).X = 1
    Vectors(2).Y = 0
    
    Vectors(3).X = 1
    Vectors(3).Y = 1
    
    
    ' Mark to search for
    CMark = Board(OriginX, OriginY).Player
    
    If WinDebug = True Then modBoard.Draw
    
    ' Search through all vectors
    FullRowSearch.Valid = False
    For Vector = 0 To 3
    
        ' Current vector
        VecX = Vectors(Vector).X
        VecY = Vectors(Vector).Y
    
        ' Default location (origin - -Vector)
        CX = OriginX - -(VecX * WinCells)
        CY = OriginY - -(VecY * WinCells)
        
        ' Reset
        RowCount = 0

        ' Search
        For A = 1 To (WinCells * 2) - 1
            
            ' Advance one step
            CX = CX + -VecX
            CY = CY + -VecY
            
            ' Check for boundaries
            If CX < 0 Or CX > BoardWidth - 1 Then GoTo NextCell
            If CY < 0 Or CY > BoardHeight - 1 Then GoTo NextCell
            
            If WinDebug = True Then modBoard.DrawHighlight CX, CY, ColorHighlight
            
            ' See if this cell is occupied and what mark it has
            If Board(CX, CY).Occupied = True And Board(CX, CY).Player = CMark Then
                RowCount = RowCount + 1
                If WinDebug = True Then modBoard.DrawHighlight CX, CY, RGB(255, 255, 255)
            Else
                RowCount = 0
            End If
            
            ' See if we have all the required cells for a win
            If RowCount = WinCells Then Exit For
            
NextCell:
        Next A
        
        ' See if we found enough contiguous marks
        If RowCount = WinCells Then
            
            ' Set return values
            With FullRowSearch
                .X = Vectors(Vector).X
                .Y = Vectors(Vector).Y
                .Valid = True
                .OriginX = CX
                .OriginY = CY
            End With
            
            Exit Function
        End If
    
    Next Vector
    
End Function
