Attribute VB_Name = "modNet"
Option Explicit
Option Compare Binary
Option Base 0


' Connection settings
Public RemoteIP As String
Public ServerPort As Long
Public ClientPort As Long
Public LocalName As String
Public ConnectionActive As Boolean

' Variable transferring
Private GotVar As String

' Socket control to use
Private sckSocket As Winsock

' Command constants
Public Enum NetCmdEnum
    CMD_NAME
    CMD_SETMARK
    CMD_CHAT
    CMD_CHATDO
    CMD_END
    CMD_DISCONNECT
    CMD_REQUESTVAR
End Enum

' Variable constants
Public Enum NetVarEnum
    VAR_SEED
    VAR_NAME
End Enum


' Initialise the net module
Public Sub Init(ByRef sckPassSocket As Winsock)

    ' Set socket control reference
    Set sckSocket = sckPassSocket
    
    modLog.WriteMsg "** Net module initialised."

End Sub


' Connect!
Public Sub Connect()

    ' Determine method of connection
    If GameType = GameServer Then
        modLog.WriteMsg "** Pressing ESC will end the listening attempt."
        modLog.WriteMsg "** Listening for incoming connection at port " & ServerPort & "..."
        
        ' Listen for incoming connection
        sckSocket.LocalPort = ServerPort
        sckSocket.Listen
        
        frmMain.Suspend
        Do
            DoEvents
        Loop Until ConnectionActive = True
        frmMain.Continue
        
    ElseIf GameType = GameClient Then
        modLog.WriteMsg "** Pressing ESC will end the connection try."
        modLog.WriteMsg "** Connecting to " & ClientPort & ":" & RemoteIP & "..."
        
        ' Try to establish a connection
        sckSocket.RemoteHost = RemoteIP
        sckSocket.RemotePort = ClientPort
        sckSocket.Close
        sckSocket.Connect
        
        frmMain.Suspend
        Do
            DoEvents
        Loop Until sckSocket.State = sckConnected
        frmMain.Continue
        modLog.WriteMsg "** Connection established."
        
        modNet.SendCommand CMD_GETSEED
        
    End If

End Sub


' Respond to an incoming connection request
Public Sub ConnectionRequest(ByVal requestID As Long)

    ' Accept the connection
    sckSocket.Close
    sckSocket.Accept requestID
    
    ' We now have a proper connection
    ConnectionActive = True
    modLog.WriteMsg "** Accepted incoming connection."

End Sub


' Just send a string of raw data
Public Sub SendRaw(ByRef Message As String)

    sckSocket.SendData Message

End Sub


' Process arrived data
Public Sub DataArrival(ByRef Data As String)

    Dim CmdNum As Integer
    
    
    ' See what command it is
    CmdNum = Val(Left$(Data, InStr(1, Data, " ")))
    
    ' Act upon command
    Select Case CmdNum
    
        ' Socket requested a variable (client)
        Case CMD_REQUESTVAR
            
            Parm = Trim$(Mid$(Data, InStr(1, Data, " ") + 1))
            
            If Parm = VAR_SEED Then SendDat = RandomSeed
            If Parm = VAR_NAME Then SendDat = "Named"
            
            modNet.SendCommand CMD_SENDVAR, SendDat
            
        ' Socket sent variable (server)
        Case CMD_SENDVAR
            GotVar = Trim$(Mid$(Data, InStr(1, Data, " ") + 1))
    
    End Select

End Sub


' Send a command to the socket
Public Sub SendCommand(ByVal CmdNum As Integer, Parameters As String)

    Dim SendString As String
    
    
    ' Create a string out of the data
    SendString = Trim$(Str$(CmdNum) & " " & Parameters)
    
    ' Send it
    modNet.SendRaw SendString

End Sub


' Get a variable from the socket
Public Function GetVar(ByVal VarNum As Integer) As String

    ' Send the request
    modNet.SendCommand CMD_REQUESTVAR, VarNum
    
    ' Wait until the variable arrived
    Do
        DoEvents
    Loop Until GotVar <> ""
    GotVar = ""
    
    ' Return it
    GetVar = GotVar

End Function
